#!/bin/sh
########################################################################
##
##                          WEIDU INSTALLER v2
##                              by Loriel
##
##  In addition to installing WeiDU, this script gives the options for
##  installing sox, tispack (and tisunpack), and a syntax highlighter
##  for TextWrangler and/or BBEdit.  It also allows you to update all
##  WeiDU executables in a given game folder named "setup-foo".  It does
##  not matter whether it is named with upper,lower, or mixed case.  The
##  executables are checked for version, and it will display the version
##  number so you don't accidentally overwrite a newer version.
##
##  Future compilers of WeiDU are free to edit this script as necessary,
##  but please give me credit (and it would be nice if you let me know).
##  Also, please (please) comment your additions thoroughly (and give
##  yourself the credit) so that anyone that comes after you knows what
##  you were thinking.
##
##  For this script's version history, please refer to the included
##  'README-Installer Script' file.
##
##  For the time being, the only things that may need to be changed are
##  the $version_installer and $version_plist variables.  They simply
##  tell the installer what versions to display for this script and the
##  syntax highlighter.  The $version_plist also sets the base version
##  to compare any installed versions to.  You should only update the
##  $version_plist if you have updated the 'install_plist' function to
##  the latest version.
##
########################################################################

version_installer="2"
version_plist="21600"

########################################################################
##
##  DEBUGGING FOR TECH SUPPORT
##
##  NOTE: As of v2, debug mode doesn't work.  It will be implemented
##        again, don't worry.  I don't relish the headaches of trying to
##        figure out what's going wrong with no type of input from you!
##
##  If you are having trouble with any part of the installation process,
##  please change the debug variable below to 1, then save and run this
##  script again.  A new file will be created called 'debug' in the same
##  folder as this script.  Please report the results of that file to me
##  by one of the following channels:
##
##  AS&G forums:
##      http://ire.mainecoon.net/yabbse/index.php
##  MacInFinity forums:
##      http://loriel.quixjote.com/forums/index.php
##  Feedback email form:
##      http://loriel.quixjote.com/index.php?page=5.0
##
########################################################################

debug=0

########################################################################
##
##  TODO LIST
##  - Command history
##  - Work in the progress bar?
##  - Figure out what to do with the $errors array
##  - Debug mode
##
########################################################################
##
##  DONE LIST
##   The whole 'update the mods' section
##   Uninstall only available if already installed
##	 Make tempfile for menus and input (instead of case statements)?
##   Better version checking - now uses the app's version number rather
##    than comparing filesize.
##
########################################################################


########################################################################
##
##  NO NEED TO EDIT BELOW THIS POINT
##
########################################################################


########################################################################
##
##  GIVE ME SOME MENUS AND RESPOND TO THEM
##  This the menu function - it determines what the end-user will see
##  when they launch the installer and make their selections.
##
########################################################################

menus() {
	clear
	tput cup 0 0
	echo
	echo "                              WeiDU Installer  v$version_installer"
	echo "                                WeiDU-mac  v$version_weidu"
	echo "                      website: http://loriel.quixjote.com/"
	echo
	blank="                                                                                "
	tput cup 5 0
	for ((i=0; $i<13; i++)); do
		echo "$blank"
	done
	tput cup 5 0
	prompt_user="Select an option: "
	menus_tempfile $1 $2 $3 $4 $5
	case "$1" in
		WELCOME)				command_history[$command]="Go to 'Welcome' menu"; let command++
								echo "Welcome to the Installer for WeiDU-mac!  As of v2, the WeiDU Installer includes"
								echo "optional installs of tispack, sox, and a syntax highlighter for TextWrangler or"
								echo "BBEdit.  To get started, please choose an option from the list below."
								echo
								line_number=8;;
		WEIDU|SOX|TIS|PLIST)	case "$1" in
									WEIDU)	command_history[$command]="Go to 'Install WeiDU' menu"; let command++;;
									SOX)	command_history[$command]="Go to 'Install Sox' menu"; let command++;;
									TIS)	command_history[$command]="Go to 'Install Tispack' menu"; let command++;;
									PLIST)	command_history[$command]="Go to 'Install Highlighter' menu"; let command++;;
								esac
								line_number=6;;
		UPDATE)					command_history[$command]="Go to 'Update WeiDU mods' menu"; let command++
								echo "Please either browse to an IE game folder, or just allow this script to search"
								echo "your computer for any valid IE games."
								echo
								line_number=7;;
		UPD_REST)				command_history[$command]="Please select an IE game folder"; let command++
								echo "Please select whether you would like to update these mods or restore them from"
								echo "the most recent backup."
								echo
								line_number=7;;
		PASSWORD)				command_history[$command]="Please enter an Admin password"; let command++
								echo "In order to install/uninstall these tools, you must provide an Admin password."
								echo "If you are the only user on this computer, your password is most likely it."
								echo
								prompt_user="Enter a password: "
								line_number=7;;
		ERR_PASS)				command_history[$command]="Incorrect password"; let command++
								echo "Oops!  That password is incorrect.  Would you like to try again?"
								echo
								line_number=6;;
		QUIT)					echo
								echo
								echo "          Thanks for using the WeiDU Installer $version_installer. Happy gaming!"
								echo
								echo
								command_history[$command]="Quitting now...press <ENTER> to continue."; let command++
								prompt_user="Please press <ENTER> to quit."
								line_number=9;;
		OOPS)					tput cup 21 0
								echo "$blank"
								tput cup 21 0
								echo "Oops! Please try again!  Press <ENTER> to continue."
								for ((i=2,x=$command;$i>0; i--,x--)); do
									tput cup $((18+$i)) 0
									echo "$blank"
									if [[ $i -gt 0 && $x -le ${#command_history[@]} ]]; then
										tput cup $((18+$i)) 0
										echo "${command_history[$x]}"
									fi
								done
								read OOPS
								menus $2;;
		*)						;;
	esac
	i=1
	clear_array choose enduser
	while read line; do
		case "$1" in
			PASSWORD|QUIT)	let i--; choose[$i]="$line";;
			*)				i=`printf "$line" | grep "^[0-9]\{1,\}\|" | awk -F \| '{print $1}'`
							if [ $i -gt 0 ]; then
								choose[$i]="`printf \"$line\" | grep \"^$i\|\" | awk -F \| '{print $2}'`"
								enduser[$i]="`printf \"$line\" | grep \"^$i\|\" | awk -F \| '{print $3, $4}'`"
							fi;;
		esac
	done < "$tmp""_menus"
	case "$1" in
		PASSWORD|QUIT)	;;
		*)				for ((i=1,x=0;$x<${#choose[@]};i++,x++,line_number++)); do
							if [ "`grep \"^$i\|\" \"$tmp\"\"_menus\"`" ]; then
								printf "%22s" "$i)  "; echo "${enduser[$i]}"
							fi
						done;;
	esac
	printf "%"$((20+${#prompt_user}))"s" "$prompt_user"
	echo
	for ((i=$line_number; $i<18; i++)); do
		echo
	done
	tput cup 18 0
	echo "================================================================================"
	for ((i=3,x=$(($command-1)); $i>0; i--,x--)); do
		tput cup $((18+$i)) 0
		echo "$blank"
		if [[ $i -gt 0 && $x -ge 0 ]]; then
			tput cup $((18+$i)) 0
			echo ${command_history[$x]}
		fi
	done
	tput cup $(($line_number+1)) $((${#prompt_user}+20))
	if [ "$1" == "QUIT" ]; then
		read QUIT
		exit
	fi
	user_input $1 $2 $3 $4 $5
}

##  This reads the user input from each menu and then redirects to the
##  proper function.

user_input() {
	read INPUT
	case "$1" in
		PASSWORD|QUIT)	for ((i=0;$i<${#choose[@]};i++)); do
							eval "${choose[$i]}"
						done;;
		*)				case "$INPUT" in
							[0-9]*)	if [ $INPUT -le ${#choose[@]} ]; then
										input_command="${choose[$INPUT]}"
									else
										input_command="menus OOPS $1"
									fi
									eval "$input_command";;
							*)		menus OOPS $1;;
						esac;;
	esac
}

menus_tempfile() {
	case "$1" in
		WELCOME)				echo "1|menus WEIDU|Install/Uninstall WeiDU" > "$tmp""_menus"
								echo "2|menus SOX|Install/Uninstall Sox" >> "$tmp""_menus"
								echo "3|menus TIS|Install/Uninstall Tisunpack" >> "$tmp""_menus"
								echo "4|menus PLIST|Install/Uninstall Syntax Highlighter" >> "$tmp""_menus"
								echo "5|menus UPDATE|Update/Restore Mods" >> "$tmp""_menus"
								echo "6|menus QUIT|Quit" >> "$tmp""_menus";;
		WEIDU|SOX|TIS|PLIST)	menu_builder $1;;
		UPDATE)					echo "1|update_mods_prep 0; menus WELCOME|Search computer for IE games" > "$tmp""_menus"
								echo "2|update_mods_prep 1; menus WELCOME|Browse to IE game folder" >> "$tmp""_menus"
								echo "3|menus WELCOME|Main Menu" >> "$tmp""_menus"
								echo "4|menus QUIT|Quit" >> "$tmp""_menus";;
		UPD_REST)				echo "1|update_mods_now|Update mods to WeiDU v$version_weidu" > "$tmp""_menus"
								echo "2|restore_mods_now|Restore mod(s) to last backup" >> "$tmp""_menus"
								echo "3|menus WELCOME|Main Menu" >> "$tmp""_menus"
								echo "4|menus QUIT|Quit" >> "$tmp""_menus";;
		PASSWORD)				echo "check_fail=\`check_password \$INPUT\`; if [ \$check_fail -eq 1 ]; then menus ERR_PASS \$2 \$3 \$4; else install_stuff \$2 \$3 \$INPUT \$4; fi" > "$tmp""_menus";;
		ERR_PASS)				echo "1|menus PASSWORD|Enter Password" > "$tmp""_menus"
								echo "2|menus WELCOME|Main Menu" >> "$tmp""_menus"
								echo "3|menus QUIT|Quit" >> "$tmp""_menus";;
		QUIT)					echo "menus QUIT" > "$tmp""_menus";;
		*)						;;
	esac
}

########################################################################
##
##  WHERE DO THINGS GO?
##  This checks which version of Mac OS X is being used, then checks for
##  valid install locations based on the results.  A valid location will
##  require that the directory structure exists, and that it is listed
##  in the $PATH variable.  If more than one valid install directory is
##  found, the user will be prompted to install to either or both of the
##  locations.
##
##  EXAMPLES:
##      10.4.x --> /usr/local/bin
##             --> /sw/bin
##      10.3.x --> /usr/local/bin
##             --> /sw/bin
##      10.2.x --> /sw/bin
##      10.1.x --> /bin
##      10.0.x --> /bin
##
########################################################################

where_do_things_go() {
	clear_array temp paths
	osx_vers=`sw_vers -productVersion | awk -F . '{print $2}'`
	i=0
	case "$osx_vers" in
		4|3)	panther_tiger;;
		2)		jaguar;;
		*)		puma_cheetah;;
	esac
	for ((i=0,x=0;$i<${#temp[@]};i++)); do
		if [ "`printf $PATH | grep ${temp[$i]}`" ]; then
			paths[$x]="${temp[$i]}"
			let x++
		fi
	done
}

panther_tiger() {
	if [ -d "/usr/local/bin" ]; then
		temp[$i]="/usr/local/bin"
		let i++
	fi
	jaguar
}

jaguar() {
	if [ -d "/sw/bin" ]; then
		temp[$i]="/sw/bin"
		let i++
	fi
	puma_cheetah
}

puma_cheetah() {
	if [[ ${#temp[@]} -eq 0 && -d "/bin" ]]; then
		temp[$i]="/bin"
		let i++
	fi
}

########################################################################
##
##  CHECK PASSWORD
##  Checks if sudo has already been successfully invoked.  If it has,
##  then it doesn't matter what the password is because the user is
##  considered a superuser for the time being, so we continue to doing
##  the actual operation (install/uninstall) by returning 0.  Otherwise,
##  the user must supply a password first, so we return a 1.
##
########################################################################

check_password() {
	printf "$1" | sudo -S -p '' /bin 2>/tmp/weidu_check
	local temp=`cat /tmp/weidu_check | grep 'incorrect password attempt'`
	if [ "${#temp}" -gt 0 ]; then
		printf 1
	else
		printf 0
	fi
}

########################################################################
##
##  CHECK VERSION
##  Takes one argument: the file that you want to find the version of.
##  This function is designed to be used like so:
##
##  version=`check_version '/usr/local/bin/weidu-mac'`
##
########################################################################

check_version() {
	local temp=`printf "$1" | sed s="^.*/"==g | tr '[A-Z]' '[a-z]'`
	local temp_dir=`printf "$1" | sed s="/[A-Za-z0-9\.\_\-]*$"==g`
	case "$temp" in
		weidu-mac)		yes | "$1" 2>/dev/null > /tmp/weidu_versions; grep "^\[$1\] WeiDU version [0-9]\{1,\}$" /tmp/weidu_versions | sed s="^\[.*\] "==g | awk '{print $3}';;
		setup-*)		cp "$1" "$temp_dir/weidu-mac"; chmod 777 "$temp_dir/weidu-mac"; yes | "$temp_dir/weidu-mac" 2>/dev/null > /tmp/weidu_versions; grep "^\[$temp_dir/weidu-mac\] WeiDU version [0-9]\{1,\}$" /tmp/weidu_versions | sed s="^\[.*\] "==g | awk '{print $3}';;
		sox)			"$1" -V 2>/tmp/weidu_versions; grep "^.*/sox:" /tmp/weidu_versions | sed s="^.*\: "==g | awk '{print $2}';;
		tispack)		"$1" -V 2>/dev/null > /tmp/weidu_versions; grep "^tispack [0-9]\{1,\}" /tmp/weidu_versions | awk '{print $2}';;
		tisunpack)		"$1" -V 2>/dev/null > /tmp/weidu_versions; grep "^tisunpack [0-9]\{1,\}" /tmp/weidu_versions | awk '{print $2}';;
		weidu.plist)	if [ -e "$1" ]; then temp=`grep "<key>CFBundleVersion</key> <string>[0-9]\{1,\}</string>" "$1" | sed s="^.*<string>"==g | sed s="</string>.*$"==g`; if [ "$temp" ]; then printf "$temp"; else printf '1'; fi; elif [ ! -d "$temp_dir" ]; then printf "$version_plist"; else printf '0'; fi;;
	esac
}

########################################################################
##
##  IS IT INSTALLED?
##  Takes two arguments: number of items in the $paths array and the
##  filename.  The results is the $inst array that holds the filesize of
##  any installed files that were found.  The filesize is used later to
##  determine if it is a newer/older version.
##
########################################################################

is_it_installed() {
	clear_array vers
	if [ -e "$1/$2" ]; then
		vers=`check_version "$1/$2"`
		uninstall[$3]=1
	else
		temp=0
		uninstall[$3]=0
	fi
}

########################################################################
##
##  UTILITY FUNCTIONS
##  These functions do boring but important stuff like clearing arrays,
##  installing stuff, etc.
##
########################################################################

##  Declares or clears an array or variable so it can be used again
##  without the previous values.

clear_array() {
	for file in "$@"; do
		eval "$file=()"
	done
}

##  Sets up the display for the various install menus.  This way I don't
##  have to repeat the code a ton of times.

menu_builder() {
	case "$1" in
		WEIDU)	local item='weidu-mac'; local current=`check_version "$command_path/weidu-mac"`; local display="WEIDU VERSIONS"; local s=52; printf "%54s" "CURRENT WEIDU VERSION:  v$current"; echo; echo;;
		SOX)	local item='sox'; local current=`check_version "$command_path/sox"`; local display="SOX VERSIONS"; local s=51; printf "%55s" "CURRENT SOX VERSION:  v$current"; echo; echo;;
		TIS)	local item='tispack'; local current=`check_version "$command_path/tispack"`; local display="TISPACK VERSIONS"; local s=53; printf "%55s" "CURRENT TISPACK VERSION: v$current"; echo; echo;;
		PLIST)	local item='WeiDU.plist'; local current=`check_version "WeiDU.plist"`; local display="HIGHLIGHTER VERSIONS"; local s=55; printf "%57s" "CURRENT HIGHLIGHTER VERSION:  v$current"; echo; echo;;
	esac
	if [ "$1" == "PLIST" ]; then
		i=0
		if [ -d "$HOME/Library/Application Support/BBEdit/Language Modules" ]; then part[$i]="$item for BBEdit"; locations[$i]="$HOME/Library/Application Support/BBEdit/Language Modules"; let i++; fi
		if [ -d "$HOME/Library/Application Support/TextWrangler/Language Modules" ]; then part[$i]="$item for TextWrangler"; locations[$i]="$HOME/Library/Application Support/TextWrangler/Language Modules"; let i++; fi
	else
		for ((i=0;$i<${#paths[@]};i++)); do
			locations[$i]="${paths[$i]}"
			part[$i]="${paths[$i]}/$item"
		done
	fi
	if [ -e "$tmp""_menus" ]; then rm -f "$tmp""_menus"; fi
	if [ "`printf \"$current\" | grep '\.'`" ]; then current=`printf "$current" | sed s="\."==g`; fi
	while [ "`printf \"$current\" | grep \"^0\"`" ]; do current=`printf "$current" | sed s="^0"==g`; done
	for ((i=0,x=1,y=$((${#locations[@]}+1));$i<${#locations[@]};i++,x++)); do
		if [[ ${#locations[@]} -eq 2 && $i -eq 0 ]]; then let y++; fi
		is_it_installed "${locations[$i]}" "$item" "$i"
		temp=$vers
		if [ "`printf \"$vers\" | grep '\.'`" ]; then temp=`printf "$vers" | sed s="\."==g`; fi
		case `check_password 0` in
			0)	while [ "`printf \"$temp\" | grep \"^0\"`" ]; do temp=`printf "$temp" | sed s="^0"==g`; done
				if [[ $current -ge $temp && $temp -gt 1 ]]; then
					echo "$x|install_stuff 1 $1 0 $i|Reinstall ${part[$i]} (v$vers)" >> "$tmp""_menus"
					echo "$y|install_stuff 0 $1 0 $i|Uninstall ${part[$i]} (v$vers)" >> "$tmp""_menus"
					let y++
				elif [[ $current -ge $temp && $temp -eq 1 ]]; then
					echo "$x|install_stuff 1 $1 0 $i|Reinstall ${part[$i]} (unknown version)" >> "$tmp""_menus"
					echo "$y|install_stuff 0 $1 0 $i|Uninstall ${part[$i]} (unknown version)" >> "$tmp""_menus"
					let y++
				elif [[ $current -lt $temp && $temp -gt 0 ]]; then
					echo "$x|install_stuff 1 $1 0 $i|Reinstall ${part[$i]} (v$vers) (not recommended)" >> "$tmp""_menus"
					echo "$y|install_stuff 0 $1 0 $i|Uninstall ${part[$i]} (v$vers) (not recommended)" >> "$tmp""_menus"
					let y++
				else
					echo "$x|install_stuff 1 $1 0 $i|Install ${part[$i]} (not installed)" >> "$tmp""_menus"
				fi;;
			1)	while [ "`printf \"$temp\" | grep \"^0\"`" ]; do temp=`printf "$temp" | sed s="^0"==g`; done
				if [[ $current -ge $temp && $temp -gt 1 ]]; then
					echo "$x|menus PASSWORD 1 $1 $i|Reinstall ${part[$i]} (v$vers)" >> "$tmp""_menus"
					echo "$y|menus PASSWORD 0 $1 $i|Uninstall ${part[$i]} (v$vers)" >> "$tmp""_menus"
					let y++
				elif [[ $current -ge $temp && $temp -eq 1 ]]; then
					echo "$x|menus PASSWORD 1 $1 $i|Reinstall ${part[$i]} (unknown version)" >> "$tmp""_menus"
					echo "$y|menus PASSWORD 0 $1 $i|Uninstall ${part[$i]} (unknown version)" >> "$tmp""_menus"
					let y++
				elif [[ $current -lt $temp && $temp -gt 0 ]]; then
					echo "$x|menus PASSWORD 1 $1 $i|Reinstall ${part[$i]} (v$vers) (not recommended)" >> "$tmp""_menus"
					echo "$y|menus PASSWORD 0 $1 $i|Uninstall ${part[$i]} (v$vers) (not recommended)" >> "$tmp""_menus"
					let y++
				else
					echo "$x|menus PASSWORD 1 $1 $i|Install ${part[$i]} (not installed)" >> "$tmp""_menus"
				fi;;
		esac
	done
	if [ ${#locations[@]} -eq 2 ]; then
		echo "3|menus PASSWORD 1 $1 2|(Re)Install both" >> "$tmp""_menus"
		if [[ -e "${locations[0]}/$item" || -e "$locations[1]/$item" ]]; then
			echo "$y|menus PASSWORD 0 $1 $i|Uninstall both" >> "$tmp""_menus"
			let y++
		fi
	fi
	echo "$y|menus WELCOME|Main Menu" >> "$tmp""_menus"; let y++
	echo "$y|menus QUIT|Quit" >> "$tmp""_menus"
}

##  INSTALL/UNINSTALL STUFF (WEIDU/SOX/TISPACK/TISUNPACK)
##  Takes 4 arguments:
##  $1  Integer representing (un)install (ie: 1=install)
##  $2  Refering menu (ie: WEIDU)
##  $3  Password INPUT
##	$4  integer representing which $path to install into

install_stuff() {
	install_switch="$1"
	case "$2" in
		WEIDU)	items=('weidu-mac');;
		SOX)	items=('sox');;
		TIS)	items=('tispack' 'tisunpack');;
		PLIST)	install_plist $1 $4;;
	esac
	if [ "$2" != "PLIST" ]; then
		password="$3"
		case "$4" in
			0)	locations=("${locations[0]}");;
			1)	locations=("${locations[1]}");;
			2)	locations=("${locations[0]}" "${locations[1]}");;
		esac
		x=0
		for item in ${items[@]}; do
			for ((i=0;$i<${#locations[@]};i++)); do
				if [[ "$install_switch" == 1 && -d "${locations[$i]}" && -e "$item" ]]; then
					printf "$password" | sudo -S -p '' cp "$item" "${locations[$i]}"  2>/tmp/weidu_install_errors
					command_history[$command]="Installing ${locations[$i]}/$item"
					let command++
				fi
				if [[ "$install_switch" == 0 && -e "${locations[$i]}/$item" ]]; then
					printf "$password" | sudo -S -p '' rm -f "${locations[$i]}/$item" 2>/tmp/weidu_install_errors
					command_history[$command]="Uninstalling ${locations[$i]}/$item"
					let command++
				fi
				if [ -e "/tmp/weidu_install_errors" ]; then
					temp=`cat /tmp/weidu_install_errors | grep 'incorrect password attempt'`
					if [ "$temp" ]; then
						errors[$x]="$temp"
						let x++
					fi
					temp=`cat /tmp/weidu_install_errors | grep 'No such file or directory'`
					if [ "$temp" ]; then
						errors[$x]="$temp"
						let x++
					fi
				fi
			done
		done
		if [ "${errors[@]}" ]; then
			echo "The following errors occurred:"
			for ((i=0;$i<${#errors[@]};i++)); do
				echo "${errors[$i]}"
			done
		fi
	fi
	menus WELCOME
}

##  UPDATE MODS PREP
##  This selects which IE game folder to affect and takes 1 argument:
##  $1  0='Search computer for games' or 1='Browse to IE game folder'

update_mods_prep() {
	case "$1" in
		0) 	clear_array temp keys list
			temp=(`ls /Volumes | tr ' ' '\134'`)
			for ((i=0,x=0;$i<${#temp[@]};i++)); do
				temp[$i]="`printf \"/Volumes/${temp[$i]}\" | tr '\134' ' '`"
				find "${temp[$i]}" -type f -iname "chitin\.key" 2>/dev/null | sed s=//=/=g > "$tmp"
				while read line; do
					case "$line" in
						*[Bb][Aa][Cc][Kk][Uu][Pp]*)				;;
						*[Tt][Uu][Tt][Uu][Cc][Oo][Pp][Yy]*)		;;
						*[Gg][Aa][Mm][Ee]XjmX[Dd][Aa][Tt][Aa]*)	list[$x]='"'"`printf \"$line\" | sed s=/[Cc][Hh][Ii][Tt][Ii][Nn]\.[Kk][Ee][Yy]==g`"'"'; let x++;;
						*[Cc][Hh][Ii][Tt][Ii][Nn]*)				list[$x]='"'"`printf \"$line\" | sed s=/[Cc][Hh][Ii][Tt][Ii][Nn]\.[Kk][Ee][Yy]==g`"'"'; let x++;;
						*)										;;
					esac
				done < "$tmp"
			done
			clear_array temp
			for ((i=0,x=$((${#list[@]}-1));$i<${#list[@]};i++)); do
				if [ $i -eq $x ]; then
					temp="$temp`printf \"${list[$i]}\"`"
				else
					temp="$temp`printf \"${list[$i]}, \"`"
				fi
			done
			msg="Please choose an IE game folder:"
			game=`osascript -e "tell application \"Terminal\" to return (choose from list {$temp} with prompt \"$msg\" ok button name \"Continue\" cancel button name \"Cancel\")"`
			command_history[$command]="Find IE games"; let command++;;
		1)	game=`osascript -e 'tell application "Terminal" to return (POSIX path of (choose folder with prompt "Please choose an IE game folder:"))'`
			command_history[$command]="Browse to IE game"; let command++;;
	esac
	if [[ -d "$game" && "`ls \"$game\" | grep \"[Cc][Hh][Ii][Tt][Ii][Nn]\.[Kk][Ee][Yy]\"`" ]]; then
		update_mods_list
	else
		command_history[$command]="Cancelling..."; let command++
	fi
}

##  UPDATE MODS LIST
##  Creates a list of mods for the user to select from.

update_mods_list() {
	clear_array mods disp
	while [ "`printf \"$game\" | grep \"/$\"`" ]; do game="`printf \"$game\" | sed s=\"/$\"==g`"; done
	mods=(`ls "$game" | egrep -i "^(setup|weidu)-[a-z0-9_-]+(\.exe)?$"`)
	for ((i=0,x=0;$i<${#mods[@]};i++)); do
		clear_array temp
		temp=`printf "${mods[$i]}" | tr '[A-Z]' '[a-z]' | grep 'setup-'`
		temp=`check_version "$game/${mods[$i]}"`
		if [ -e "$game/weidu-mac" ]; then
			rm -f "$game/weidu-mac"
		fi
		if [ -e "$game/${mods[$i]}" ]; then
			if [ "`printf \"${mods[$i]}\" | grep -i '.exe'`" ]; then
				disp[$x]='"'"${mods[$i]} (PC version $temp)"'"'
			elif [ $temp -gt $version_weidu ]; then
				disp[$x]='"'"${mods[$i]} (v$temp) (not recommended)"'"'
			else
				disp[$x]='"'"${mods[$i]} (v$temp)"'"'
			fi
			let x++
		fi
	done
	clear_array temp
	for ((i=0,x=$((${#disp[@]}-1));$i<${#disp[@]};i++)); do
		if [ $i -eq $x ]; then
			temp="$temp`printf \"${disp[$i]}\"`"
		else
			temp="$temp`printf \"${disp[$i]}, \"`"
		fi
	done
	clear_array mods
	msg="Current version of WeiDU is v$version_weidu.\nPlease choose one or more mods to update:"
	mods=(`osascript -e "tell application \"Terminal\" to return (choose from list {$temp} with prompt \"$msg\" ok button name \"Continue\" cancel button name \"Cancel\" with multiple selections allowed)" | sed s="\ ([A-Za-z0-9\ ]*)"==g | sed s=','==g`)
#	echo "${mods[@]}"
	if [[ ${#mods[@]} -gt 0 && "${mods[0]}" != "false" ]]; then
		menus UPD_REST
	else
		command_history[$command]="Cancelling..."; let command++
	fi
}

##  UPDATE MODS NOW
##  Cycles through the previously determined list of mods and updates
##  them to this version of WeiDU.
##  1)  If the backup folder doesn't exist, make it
##  2)  Backup the mod executable in the backup folder without overwriting.
##  3)  Update the mod's executable to this version of WeiDU.

update_mods_now() {
	command_history[$command]="Updating mods to WeiDU v$version_weidu"; let command++
	if [ ! -d "$game/_jmbackup/WeiDU" ]; then
		mkdir -pm 777 "$game/_jmbackup/WeiDU"
	fi
	for ((i=0;$i<${#mods[@]};i++)); do
		if [ "`printf \"${mods[$i]}\" | grep -i \"\.exe$\"`" ]; then
			weidu_file=`printf "${mods[$i]}" | sed s='\.[Ee][Xx][Ee]'==g`
			original_file="${mods[$i]}"
			if [ -e "$game/_jmbackup/WeiDU/${mods[$i]}" ]; then
				backup_file="$weidu__file"
			else
				backup_file="$original_file"
			fi
		else
			weidu_file="${mods[$i]}"
			backup_file="${mods[$i]}"
			original_file="${mods[$i]}"
		fi
		x=0
		while [[ -e "$game/_jmbackup/WeiDU/$backup_file" || -e "$game/_jmbackup/WeiDU/$backup_file.exe" ]]; do
			let x++
			backup_file="$original_file-$x"
		done
		if [ -e "$game/$original_file" ]; then
			mv -f "$game/$original_file" "$game/_jmbackup/WeiDU/$backup_file"
		fi
		if [ -e "weidu-mac" ]; then
			cp "weidu-mac" "$game/$weidu_file"
		fi
	done
}

##  RESTORE MODS FINAL
##  Cycles through the previously determined list of mods and restores
##  them from the latest backup in the backup folder.
##  1)  Cycles through the backups of the 

restore_mods_now() {
	command_history[$command]="Restoring mods to last backup"; let command++
	for ((i=0;$i<${#mods[@]};i++)); do
		backup_file="${mods[$i]}"
		x=0
		while [[ -e "$game/_jmbackup/WeiDU/$backup_file" || -e "$game/_jmbackup/WeiDU/$backup_file.exe" ]]; do
			let x++
			backup_file="${mods[$i]}-$x"
		done
		if [[ -e "$game/_jmbackup/WeiDU/${mods[$i]}.exe" && $x -le 1 ]]; then
			backup_file="${mods[$i]}.exe"
		elif [ $x -gt 1 ]; then
			let x--
			backup_file="${mods[$i]}-$x"
		else
			backup_file="${mods[$i]}"
		fi
		echo "$backup_file"
		case "$backup_file" in
			*.[Ee][Xx][Ee])	if [ -e "$game/_jmbackup/WeiDU/$backup_file" ]; then mv "$game/${mods[$i]}" "$game/_jmbackup/WeiDU/${mods[$i]}_bak"; mv "$game/_jmbackup/WeiDU/$backup_file" "$game/${mods[$i]}.exe"; if [[ -e "$game/${mods[$i]}.exe" && "$game/_jmbackup/WeiDU/${mods[$i]}_bak" ]]; then rm -f "$game/_jmbackup/WeiDU/${mods[$i]}_bak"; else mv "$game/_jmbackup/WeiDU/${mods[$i]}_bak" "$game/${mods[$i]}"; fi; fi;;
			*)				if [ -e "$game/_jmbackup/WeiDU/$backup_file" ]; then mv "$game/${mods[$i]}" "$game/_jmbackup/WeiDU/${mods[$i]}_bak"; mv "$game/_jmbackup/WeiDU/$backup_file" "$game/${mods[$i]}"; if [[ -e "$game/${mods[$i]}" && "$game/_jmbackup/WeiDU/${mods[$i]}_bak" ]]; then rm -f "$game/_jmbackup/WeiDU/${mods[$i]}_bak"; else mv "$game/_jmbackup/WeiDU/${mods[$i]}_bak" "$game/${mods[$i]}"; fi; fi;;
		esac
	done
}

##  INSTALL/UNINSTALL WEIDU.PLIST
##  Takes 2 arguments:
##  $1  Integer representing (un)install (ie: 1=install)
##	$2  integer representing BBEdit or TextWrangler

install_plist() {
	install_switch="$1"
	case "$2" in
		0)	locations=("${locations[0]}"); local temp=`printf "${locations[0]}" | awk '{print $2}' | sed s='Support/'==g | sed s='/Language'==g`;;
		1)	locations=("${locations[1]}"); local temp=`printf "${locations[1]}" | awk '{print $2}' | sed s='Support/'==g | sed s='/Language'==g`;;
		2)	locations=("${locations[0]}" "${locations[1]}"); local temp="Both";;
	esac
	case "$1" in
		0)	command_history[$command]="Uninstalling Syntax highlighter for $temp"; let command++;;
		1)	command_history[$command]="Installing Syntax highlighter for $temp"; let command++;;
	esac
	for ((i=0;$i<${#locations[@]};i++)); do
		plist="${locations[$i]}/WeiDU.plist"
		if [ $install_switch -eq 1 ]; then
			echo '<?xml version="1.0" encoding="UTF-8"?>' > "$plist"
			echo '<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">' >> "$plist"
			echo '<plist version="1.0">' >> "$plist"
			echo '<dict>' >> "$plist"
			echo '	<key>BBEditDocumentType</key>' >> "$plist"
			echo '	<string>CodelessLanguageModule</string>' >> "$plist"
			echo '	<key>BBLMColorsSyntax</key>' >> "$plist"
			echo '	<true/>' >> "$plist"
			echo '	<key>BBLMIsCaseSensitive</key>' >> "$plist"
			echo '	<true/>' >> "$plist"
			echo '	<key>BBLMKeywordList</key>' >> "$plist"
			echo '	<array>' >> "$plist"
			echo '		<string>A_B_P</string>' >> "$plist"
			echo '		<string>A_S_T</string>' >> "$plist"
			echo '		<string>A_T_T</string>' >> "$plist"
			echo '		<string>ABS</string>' >> "$plist"
			echo '		<string>ACTION_BASH_FOR</string>' >> "$plist"
			echo '		<string>ACTION_CLEAR_ARRAY</string>' >> "$plist"
			echo '		<string>ACTION_DEFINE_ARRAY</string>' >> "$plist"
			echo '		<string>ACTION_DEFINE_ASSOCIATIVE_ARRAY</string>' >> "$plist"
			echo '		<string>ACTION_FOR_EACH</string>' >> "$plist"
			echo '		<string>ACTION_IF</string>' >> "$plist"
			echo '		<string>ACTION_INCLUDE</string>' >> "$plist"
			echo '		<string>ACTION_PHP_EACH</string>' >> "$plist"
			echo '		<string>ACTION_READLN</string>' >> "$plist"
			echo '		<string>ACTION_REINCLUDE</string>' >> "$plist"
			echo '		<string>ADD_CRE_ITEM</string>' >> "$plist"
			echo '		<string>ADD_GAM_NPC</string>' >> "$plist"
			echo '		<string>ADD_GAME_NPC</string>' >> "$plist"
			echo '		<string>ADD_KIT</string>' >> "$plist"
			echo '		<string>ADD_KNOWN_SPELL</string>' >> "$plist"
			echo '		<string>ADD_MAP_NOTE</string>' >> "$plist"
			echo '		<string>ADD_MEMORIZED_SPELL</string>' >> "$plist"
			echo '		<string>ADD_MUSIC</string>' >> "$plist"
			echo '		<string>ADD_PROJECTILE</string>' >> "$plist"
			echo '		<string>ADD_SPELL</string>' >> "$plist"
			echo '		<string>ADD_STATE_TRIGGER</string>' >> "$plist"
			echo '		<string>ADD_STORE_ITEM</string>' >> "$plist"
			echo '		<string>ADD_TRANS_ACTION</string>' >> "$plist"
			echo '		<string>ADD_TRANS_TRIGGER</string>' >> "$plist"
			echo '		<string>AFTER</string>' >> "$plist"
			echo '		<string>ALLOW_MISSING</string>' >> "$plist"
			echo '		<string>ALTER_TLK</string>' >> "$plist"
			echo '		<string>ALTER_TLK_LIST</string>' >> "$plist"
			echo '		<string>ALTER_TLK_RANGE</string>' >> "$plist"
			echo '		<string>ALTER_TRANS</string>' >> "$plist"
			echo '		<string>ALWAYS</string>' >> "$plist"
			echo '		<string>AND</string>' >> "$plist"
			echo '		<string>AND_EQUALS</string>' >> "$plist"
			echo '		<string>APPEND</string>' >> "$plist"
			echo '		<string>APPEND_COL</string>' >> "$plist"
			echo '		<string>APPEND_EARLY</string>' >> "$plist"
			echo '		<string>APPEND_FILE</string>' >> "$plist"
			echo '		<string>APPEND_FILE_EVALUATE</string>' >> "$plist"
			echo '		<string>APPEND_OUTER</string>' >> "$plist"
			echo '		<string>APPENDI</string>' >> "$plist"
			echo '		<string>APPLY_BCS_PATCH</string>' >> "$plist"
			echo '		<string>APPLY_BCS_PATCH_OR_COPY</string>' >> "$plist"
			echo '		<string>AS</string>' >> "$plist"
			echo '		<string>ASK_EVERY_COMPONENT</string>' >> "$plist"
			echo '		<string>AT</string>' >> "$plist"
			echo '		<string>AT_EXIT</string>' >> "$plist"
			echo '		<string>AT_INTERACTIVE_EXIT</string>' >> "$plist"
			echo '		<string>AT_INTERACTIVE_NOW</string>' >> "$plist"
			echo '		<string>AT_INTERACTIVE_UNINSTALL</string>' >> "$plist"
			echo '		<string>AT_NOW</string>' >> "$plist"
			echo '		<string>AT_UNINSTALL</string>' >> "$plist"
			echo '		<string>AUTHOR</string>' >> "$plist"
			echo '		<string>AUTO_TRA</string>' >> "$plist"
			echo '		<string>BACKUP</string>' >> "$plist"
			echo '		<string>BAND</string>' >> "$plist"
			echo '		<string>BASR</string>' >> "$plist"
			echo '		<string>BEFORE</string>' >> "$plist"
			echo '		<string>BEGIN</string>' >> "$plist"
			echo '		<string>BIFF</string>' >> "$plist"
			echo '		<string>BLSL</string>' >> "$plist"
			echo '		<string>BLSL_EQUALS</string>' >> "$plist"
			echo '		<string>BLSR</string>' >> "$plist"
			echo '		<string>BLSR_EQUALS</string>' >> "$plist"
			echo '		<string>BNOT</string>' >> "$plist"
			echo '		<string>BOR</string>' >> "$plist"
			echo '		<string>BRANCH</string>' >> "$plist"
			echo '		<string>BUFFER_LENGTH</string>' >> "$plist"
			echo '		<string>BUT_ONLY</string>' >> "$plist"
			echo '		<string>BUT_ONLY_IF_IT_CHANGES</string>' >> "$plist"
			echo '		<string>BXOR</string>' >> "$plist"
			echo '		<string>BXOR_EQUALS</string>' >> "$plist"
			echo '		<string>BYTE_AT</string>' >> "$plist"
			echo '		<string>CASE_INSENSITIVE</string>' >> "$plist"
			echo '		<string>CASE_SENSITIVE</string>' >> "$plist"
			echo '		<string>CHAIN</string>' >> "$plist"
			echo '		<string>CHAIN2</string>' >> "$plist"
			echo '		<string>CHAIN3</string>' >> "$plist"
			echo '		<string>CLEAR_ARRAY</string>' >> "$plist"
			echo '		<string>CLEAR_IDS_MAP</string>' >> "$plist"
			echo '		<string>CLEAR_MEMORY</string>' >> "$plist"
			echo '		<string>COLON</string>' >> "$plist"
			echo '		<string>COMPILE</string>' >> "$plist"
			echo '		<string>COMPILE_BAF_TO_BCS</string>' >> "$plist"
			echo '		<string>COMPILE_D_TO_DLG</string>' >> "$plist"
			echo '		<string>COMPRESS_INTO_FILE</string>' >> "$plist"
			echo '		<string>COMPRESS_INTO_VAR</string>' >> "$plist"
			echo '		<string>COMPRESS_REPLACE_FILE</string>' >> "$plist"
			echo '		<string>COPY</string>' >> "$plist"
			echo '		<string>COPY_ALL_GAM_FILES</string>' >> "$plist"
			echo '		<string>COPY_EXISTING</string>' >> "$plist"
			echo '		<string>COPY_EXISTING_REGEXP</string>' >> "$plist"
			echo '		<string>COPY_KIT</string>' >> "$plist"
			echo '		<string>COPY_LARGE</string>' >> "$plist"
			echo '		<string>COPY_RANDOM</string>' >> "$plist"
			echo '		<string>COPY_TRANS</string>' >> "$plist"
			echo '		<string>COPY_TRANS_LATE</string>' >> "$plist"
			echo '		<string>COUNT_2DA_COLS</string>' >> "$plist"
			echo '		<string>COUNT_2DA_ROWS</string>' >> "$plist"
			echo '		<string>COUNT_REGEXP_INSTANCES</string>' >> "$plist"
			echo '		<string>DECOMPILE_BCS_TO_BAF</string>' >> "$plist"
			echo '		<string>DECOMPILE_DLG_TO_D</string>' >> "$plist"
			echo '		<string>DECOMPRESS_INTO_FILE</string>' >> "$plist"
			echo '		<string>DECOMPRESS_INTO_VAR</string>' >> "$plist"
			echo '		<string>DECOMPRESS_REPLACE_FILE</string>' >> "$plist"
			echo '		<string>DEFINE_ACTION_FUNCTION</string>' >> "$plist"
			echo '		<string>DEFINE_ACTION_MACRO</string>' >> "$plist"
			echo '		<string>DEFINE_ARRAY</string>' >> "$plist"
			echo '		<string>DEFINE_ASSOCIATIVE_ARRAY</string>' >> "$plist"
			echo '		<string>DEFINE_FUNCTION_ACTION</string>' >> "$plist"
			echo '		<string>DEFINE_FUNCTION_PATCH</string>' >> "$plist"
			echo '		<string>DEFINE_MACRO_ACTION</string>' >> "$plist"
			echo '		<string>DEFINE_MACRO_PATCH</string>' >> "$plist"
			echo '		<string>DEFINE_PATCH_FUNCTION</string>' >> "$plist"
			echo '		<string>DEFINE_PATCH_MACRO</string>' >> "$plist"
			echo '		<string>DELETE_BYTES</string>' >> "$plist"
			echo '		<string>DEPRECATED</string>' >> "$plist"
			echo '		<string>DESCRIBE_ITEM</string>' >> "$plist"
			echo '		<string>DESIGNATED</string>' >> "$plist"
			echo '		<string>DIVIDE</string>' >> "$plist"
			echo '		<string>DIVIDE_EQUALS</string>' >> "$plist"
			echo '		<string>DO</string>' >> "$plist"
			echo '		<string>DOLLARS</string>' >> "$plist"
			echo '		<string>EDIT_SAV_FILE</string>' >> "$plist"
			echo '		<string>ELSE</string>' >> "$plist"
			echo '		<string>END</string>' >> "$plist"
			echo '		<string>ENGINE_IS</string>' >> "$plist"
			echo '		<string>EQUALS</string>' >> "$plist"
			echo '		<string>EQUALSEQUALS</string>' >> "$plist"
			echo '		<string>EQUALSGREATER</string>' >> "$plist"
			echo '		<string>EQUIP</string>' >> "$plist"
			echo '		<string>EVAL</string>' >> "$plist"
			echo '		<string>EVALUATE_BUFFER</string>' >> "$plist"
			echo '		<string>EVALUATE_BUFFER_SPECIAL</string>' >> "$plist"
			echo '		<string>EVALUATE_REGEXP</string>' >> "$plist"
			echo '		<string>EXACT</string>' >> "$plist"
			echo '		<string>EXACT_MATCH</string>' >> "$plist"
			echo '		<string>EXIT</string>' >> "$plist"
			echo '		<string>EXTEND</string>' >> "$plist"
			echo '		<string>EXTEND_BOTTOM</string>' >> "$plist"
			echo '		<string>EXTEND_BOTTOM_REGEXP</string>' >> "$plist"
			echo '		<string>EXTEND_MOS</string>' >> "$plist"
			echo '		<string>EXTEND_TOP</string>' >> "$plist"
			echo '		<string>EXTEND_TOP_REGEXP</string>' >> "$plist"
			echo '		<string>EXTERN</string>' >> "$plist"
			echo '		<string>FAIL</string>' >> "$plist"
			echo '		<string>FILE_CONTAINS</string>' >> "$plist"
			echo '		<string>FILE_CONTAINS_EVALUATED</string>' >> "$plist"
			echo '		<string>FILE_EXISTS</string>' >> "$plist"
			echo '		<string>FILE_EXISTS_IN_GAME</string>' >> "$plist"
			echo '		<string>FILE_IS_IN_COMPRESSED_BIF</string>' >> "$plist"
			echo '		<string>FILE_MD5</string>' >> "$plist"
			echo '		<string>FILE_SIZE</string>' >> "$plist"
			echo '		<string>FIRST</string>' >> "$plist"
			echo '		<string>FLAGS</string>' >> "$plist"
			echo '		<string>FOR</string>' >> "$plist"
			echo '		<string>FORBID_COMPONENT</string>' >> "$plist"
			echo '		<string>FORBID_FILE</string>' >> "$plist"
			echo '		<string>FORCED_SUBCOMPONENT</string>' >> "$plist"
			echo '		<string>FROM</string>' >> "$plist"
			echo '		<string>GAME_IS</string>' >> "$plist"
			echo '		<string>GET_DIRECTORY_ARRAY</string>' >> "$plist"
			echo '		<string>GET_FILE_ARRAY</string>' >> "$plist"
			echo '		<string>GET_OFFSET_ARRAY</string>' >> "$plist"
			echo '		<string>GET_OFFSET_ARRAY2</string>' >> "$plist"
			echo '		<string>GET_STRREF</string>' >> "$plist"
			echo '		<string>GET_STRREF_F</string>' >> "$plist"
			echo '		<string>GET_STRREF_FS</string>' >> "$plist"
			echo '		<string>GET_STRREF_S</string>' >> "$plist"
			echo '		<string>GLOB</string>' >> "$plist"
			echo '		<string>GOTO</string>' >> "$plist"
			echo '		<string>GROUP</string>' >> "$plist"
			echo '		<string>GT</string>' >> "$plist"
			echo '		<string>GTE</string>' >> "$plist"
			echo '		<string>I_C_T</string>' >> "$plist"
			echo '		<string>I_C_T2</string>' >> "$plist"
			echo '		<string>I_C_T3</string>' >> "$plist"
			echo '		<string>I_C_T4</string>' >> "$plist"
			echo '		<string>I_S_I</string>' >> "$plist"
			echo '		<string>IDS_OF_SYMBOL</string>' >> "$plist"
			echo '		<string>IF</string>' >> "$plist"
			echo '		<string>IF_EVAL</string>' >> "$plist"
			echo '		<string>IF_FILE_EXISTS</string>' >> "$plist"
			echo '		<string>IF_SIZE_IS</string>' >> "$plist"
			echo '		<string>IN</string>' >> "$plist"
			echo '		<string>INCLUDE</string>' >> "$plist"
			echo '		<string>INNER_ACTION</string>' >> "$plist"
			echo '		<string>INNER_PATCH</string>' >> "$plist"
			echo '		<string>INNER_PATCH_FILE</string>' >> "$plist"
			echo '		<string>INNER_PATCH_SAVE</string>' >> "$plist"
			echo '		<string>INSERT_2DA_ROW</string>' >> "$plist"
			echo '		<string>INSERT_BYTES</string>' >> "$plist"
			echo '		<string>INSERT_FILE</string>' >> "$plist"
			echo '		<string>INSTALL_BY_DEFAULT</string>' >> "$plist"
			echo '		<string>INT_VAR</string>' >> "$plist"
			echo '		<string>INTERJECT</string>' >> "$plist"
			echo '		<string>INTERJECT_COPY_TRANS</string>' >> "$plist"
			echo '		<string>INTERJECT_COPY_TRANS2</string>' >> "$plist"
			echo '		<string>INTERJECT_COPY_TRANS3</string>' >> "$plist"
			echo '		<string>INTERJECT_COPY_TRANS4</string>' >> "$plist"
			echo '		<string>IS_AN_INT</string>' >> "$plist"
			echo '		<string>ITM_V10_GEN_EFFECTS</string>' >> "$plist"
			echo '		<string>ITM_V10_HEAD_EFFECTS</string>' >> "$plist"
			echo '		<string>ITM_V10_HEADERS</string>' >> "$plist"
			echo '		<string>JOURNAL</string>' >> "$plist"
			echo '		<string>KEEP_CRLF</string>' >> "$plist"
			echo '		<string>LAF</string>' >> "$plist"
			echo '		<string>LAM</string>' >> "$plist"
			echo '		<string>LANGUAGE</string>' >> "$plist"
			echo '		<string>LAST</string>' >> "$plist"
			echo '		<string>LAUNCH_ACTION_FUNCTION</string>' >> "$plist"
			echo '		<string>LAUNCH_ACTION_MACRO</string>' >> "$plist"
			echo '		<string>LAUNCH_FUNCTION_ACTION</string>' >> "$plist"
			echo '		<string>LAUNCH_FUNCTION_PATCH</string>' >> "$plist"
			echo '		<string>LAUNCH_MACRO_ACTION</string>' >> "$plist"
			echo '		<string>LAUNCH_MACRO_PATCH</string>' >> "$plist"
			echo '		<string>LAUNCH_PATCH_FUNCTION</string>' >> "$plist"
			echo '		<string>LAUNCH_PATCH_MACRO</string>' >> "$plist"
			echo '		<string>LOAD</string>' >> "$plist"
			echo '		<string>LOAD_TRA</string>' >> "$plist"
			echo '		<string>LOCAL_SET</string>' >> "$plist"
			echo '		<string>LOCAL_SPRINT</string>' >> "$plist"
			echo '		<string>LOCAL_TEXT_SPRINT</string>' >> "$plist"
			echo '		<string>LONG_AT</string>' >> "$plist"
			echo '		<string>LOOKUP_IDS_SYMBOL_OF_INT</string>' >> "$plist"
			echo '		<string>LPAREN</string>' >> "$plist"
			echo '		<string>LPF</string>' >> "$plist"
			echo '		<string>LPM</string>' >> "$plist"
			echo '		<string>LT</string>' >> "$plist"
			echo '		<string>LTE</string>' >> "$plist"
			echo '		<string>MAKE_BIFF</string>' >> "$plist"
			echo '		<string>MENU_STYLE</string>' >> "$plist"
			echo '		<string>MINUS</string>' >> "$plist"
			echo '		<string>MINUS_EQUALS</string>' >> "$plist"
			echo '		<string>MINUS_MINUS</string>' >> "$plist"
			echo '		<string>MKDIR</string>' >> "$plist"
			echo '		<string>MOD_IS_INSTALLED</string>' >> "$plist"
			echo '		<string>MODDER</string>' >> "$plist"
			echo '		<string>MOVE</string>' >> "$plist"
			echo '		<string>NO_IF_EVAL_BUG</string>' >> "$plist"
			echo '		<string>NO_LOG_RECORD</string>' >> "$plist"
			echo '		<string>NOGLOB</string>' >> "$plist"
			echo '		<string>NOT</string>' >> "$plist"
			echo '		<string>NOTEQUALS</string>' >> "$plist"
			echo '		<string>NULL</string>' >> "$plist"
			echo '		<string>OR</string>' >> "$plist"
			echo '		<string>OR_EQUALS</string>' >> "$plist"
			echo '		<string>OUTER_FOR</string>' >> "$plist"
			echo '		<string>OUTER_INNER_PATCH</string>' >> "$plist"
			echo '		<string>OUTER_INNER_PATCH_SAVE</string>' >> "$plist"
			echo '		<string>OUTER_PATCH</string>' >> "$plist"
			echo '		<string>OUTER_PATCH_SAVE</string>' >> "$plist"
			echo '		<string>OUTER_SET</string>' >> "$plist"
			echo '		<string>OUTER_SPRINT</string>' >> "$plist"
			echo '		<string>OUTER_TEXT_SPRINT</string>' >> "$plist"
			echo '		<string>OUTER_WHILE</string>' >> "$plist"
			echo '		<string>PATCH_BASH_FOR</string>' >> "$plist"
			echo '		<string>PATCH_CLEAR_ARRAY</string>' >> "$plist"
			echo '		<string>PATCH_DEFINE_ARRAY</string>' >> "$plist"
			echo '		<string>PATCH_DEFINE_ASSOCIATIVE_ARRAY</string>' >> "$plist"
			echo '		<string>PATCH_FOR_EACH</string>' >> "$plist"
			echo '		<string>PATCH_GAM</string>' >> "$plist"
			echo '		<string>PATCH_IF</string>' >> "$plist"
			echo '		<string>PATCH_INCLUDE</string>' >> "$plist"
			echo '		<string>PATCH_PHP_EACH</string>' >> "$plist"
			echo '		<string>PATCH_PRINT</string>' >> "$plist"
			echo '		<string>PATCH_RANDOM_SEED</string>' >> "$plist"
			echo '		<string>PATCH_READLN</string>' >> "$plist"
			echo '		<string>PATCH_REINCLUDE</string>' >> "$plist"
			echo '		<string>PATCH_SILENT</string>' >> "$plist"
			echo '		<string>PATCH_VERBOSE</string>' >> "$plist"
			echo '		<string>PHP_EACH</string>' >> "$plist"
			echo '		<string>PLUS</string>' >> "$plist"
			echo '		<string>PLUS_EQUALS</string>' >> "$plist"
			echo '		<string>PLUS_PLUS</string>' >> "$plist"
			echo '		<string>PRETTY_PRINT_2DA</string>' >> "$plist"
			echo '		<string>PRINT</string>' >> "$plist"
			echo '		<string>QUESTION</string>' >> "$plist"
			echo '		<string>QUOTE</string>' >> "$plist"
			echo '		<string>R_A_T</string>' >> "$plist"
			echo '		<string>R_A_T_P_R</string>' >> "$plist"
			echo '		<string>R_B_B</string>' >> "$plist"
			echo '		<string>R_B_B_RE</string>' >> "$plist"
			echo '		<string>R_S_T</string>' >> "$plist"
			echo '		<string>R_T_T</string>' >> "$plist"
			echo '		<string>RANDOM</string>' >> "$plist"
			echo '		<string>RANDOM_SEED</string>' >> "$plist"
			echo '		<string>READ_2DA_ENTRIES_NOW</string>' >> "$plist"
			echo '		<string>READ_2DA_ENTRY</string>' >> "$plist"
			echo '		<string>READ_2DA_ENTRY_FORMER</string>' >> "$plist"
			echo '		<string>READ_ASCII</string>' >> "$plist"
			echo '		<string>READ_BYTE</string>' >> "$plist"
			echo '		<string>READ_LONG</string>' >> "$plist"
			echo '		<string>READ_SBYTE</string>' >> "$plist"
			echo '		<string>READ_SHORT</string>' >> "$plist"
			echo '		<string>READ_SLONG</string>' >> "$plist"
			echo '		<string>READ_SSHORT</string>' >> "$plist"
			echo '		<string>READ_STRREF</string>' >> "$plist"
			echo '		<string>READ_STRREF_F</string>' >> "$plist"
			echo '		<string>READ_STRREF_FS</string>' >> "$plist"
			echo '		<string>READ_STRREF_S</string>' >> "$plist"
			echo '		<string>README</string>' >> "$plist"
			echo '		<string>REBUILD_CRE_FILE</string>' >> "$plist"
			echo '		<string>REINCLUDE</string>' >> "$plist"
			echo '		<string>REMOVE_2DA_ROW</string>' >> "$plist"
			echo '		<string>REMOVE_CRE_EFFECTS</string>' >> "$plist"
			echo '		<string>REMOVE_CRE_ITEM</string>' >> "$plist"
			echo '		<string>REMOVE_CRE_ITEMS</string>' >> "$plist"
			echo '		<string>REMOVE_KNOWN_SPELL</string>' >> "$plist"
			echo '		<string>REMOVE_KNOWN_SPELLS</string>' >> "$plist"
			echo '		<string>REMOVE_MEMORIZED_SPELL</string>' >> "$plist"
			echo '		<string>REMOVE_MEMORIZED_SPELLS</string>' >> "$plist"
			echo '		<string>REMOVE_STORE_ITEM</string>' >> "$plist"
			echo '		<string>REPLACE</string>' >> "$plist"
			echo '		<string>REPLACE_ACTION_TEXT</string>' >> "$plist"
			echo '		<string>REPLACE_ACTION_TEXT_PROCESS</string>' >> "$plist"
			echo '		<string>REPLACE_ACTION_TEXT_PROCESS_REGEXP</string>' >> "$plist"
			echo '		<string>REPLACE_ACTION_TEXT_REGEXP</string>' >> "$plist"
			echo '		<string>REPLACE_BCS_BLOCK</string>' >> "$plist"
			echo '		<string>REPLACE_BCS_BLOCK_REGEXP</string>' >> "$plist"
			echo '		<string>REPLACE_CRE_ITEM</string>' >> "$plist"
			echo '		<string>REPLACE_EVALUATE</string>' >> "$plist"
			echo '		<string>REPLACE_SAY</string>' >> "$plist"
			echo '		<string>REPLACE_STATE_TRIGGER</string>' >> "$plist"
			echo '		<string>REPLACE_TEXTUALLY</string>' >> "$plist"
			echo '		<string>REPLACE_TRANS_ACTION</string>' >> "$plist"
			echo '		<string>REPLACE_TRANS_TRIGGER</string>' >> "$plist"
			echo '		<string>REPLACE_TRIGGER_TEXT</string>' >> "$plist"
			echo '		<string>REPLACE_TRIGGER_TEXT_REGEXP</string>' >> "$plist"
			echo '		<string>REPLY</string>' >> "$plist"
			echo '		<string>REQUIRE_COMPONENT</string>' >> "$plist"
			echo '		<string>REQUIRE_FILE</string>' >> "$plist"
			echo '		<string>REQUIRE_PREDICATE</string>' >> "$plist"
			echo '		<string>RESPONSE</string>' >> "$plist"
			echo '		<string>RET</string>' >> "$plist"
			echo '		<string>RPAREN</string>' >> "$plist"
			echo '		<string>SAY</string>' >> "$plist"
			echo '		<string>SAY_EVALUATED</string>' >> "$plist"
			echo '		<string>SBYTE_AT</string>' >> "$plist"
			echo '		<string>SCRIPT_STYLE</string>' >> "$plist"
			echo '		<string>SEMICOLON</string>' >> "$plist"
			echo '		<string>SET</string>' >> "$plist"
			echo '		<string>SET_2DA_ENTRIES_NOW</string>' >> "$plist"
			echo '		<string>SET_2DA_ENTRY</string>' >> "$plist"
			echo '		<string>SET_2DA_ENTRY_LATER</string>' >> "$plist"
			echo '		<string>SET_BG2_PROFICIENCY</string>' >> "$plist"
			echo '		<string>SET_IDS_SYMBOL_OF_INT</string>' >> "$plist"
			echo '		<string>SET_WEIGHT</string>' >> "$plist"
			echo '		<string>SHORT_AT</string>' >> "$plist"
			echo '		<string>SILENT</string>' >> "$plist"
			echo '		<string>SLONG_AT</string>' >> "$plist"
			echo '		<string>SNPRINT</string>' >> "$plist"
			echo '		<string>SOLVED_JOURNAL</string>' >> "$plist"
			echo '		<string>SPACES</string>' >> "$plist"
			echo '		<string>SPRINT</string>' >> "$plist"
			echo '		<string>SPRINTF</string>' >> "$plist"
			echo '		<string>SSHORT_AT</string>' >> "$plist"
			echo '		<string>STATE_WHICH_SAYS</string>' >> "$plist"
			echo '		<string>STR_CMP</string>' >> "$plist"
			echo '		<string>STR_VAR</string>' >> "$plist"
			echo '		<string>STRING_COMPARE</string>' >> "$plist"
			echo '		<string>STRING_COMPARE_CASE</string>' >> "$plist"
			echo '		<string>STRING_COMPARE_REGEXP</string>' >> "$plist"
			echo '		<string>STRING_CONCAT</string>' >> "$plist"
			echo '		<string>STRING_CONTAINS_REGEXP</string>' >> "$plist"
			echo '		<string>STRING_EQUAL</string>' >> "$plist"
			echo '		<string>STRING_EQUAL_CASE</string>' >> "$plist"
			echo '		<string>STRING_LENGTH</string>' >> "$plist"
			echo '		<string>STRING_MATCHES_REGEXP</string>' >> "$plist"
			echo '		<string>STRING_SET</string>' >> "$plist"
			echo '		<string>STRING_SET_EVALUATE</string>' >> "$plist"
			echo '		<string>STRING_SET_RANGE</string>' >> "$plist"
			echo '		<string>SUB_COMPONENT</string>' >> "$plist"
			echo '		<string>SUBCOMPONENT</string>' >> "$plist"
			echo '		<string>TEXT_SPRINT</string>' >> "$plist"
			echo '		<string>THEN</string>' >> "$plist"
			echo '		<string>TIMES</string>' >> "$plist"
			echo '		<string>TIMES_EQUALS</string>' >> "$plist"
			echo '		<string>TIMESTIMES</string>' >> "$plist"
			echo '		<string>TO_LOWER</string>' >> "$plist"
			echo '		<string>TO_UPPER</string>' >> "$plist"
			echo '		<string>TWOHANDED</string>' >> "$plist"
			echo '		<string>UNINSTALL</string>' >> "$plist"
			echo '		<string>UNINSTALL_ORDER</string>' >> "$plist"
			echo '		<string>UNLESS</string>' >> "$plist"
			echo '		<string>UNSOLVED_JOURNAL</string>' >> "$plist"
			echo '		<string>USING</string>' >> "$plist"
			echo '		<string>VARIABLE_IS_SET</string>' >> "$plist"
			echo '		<string>VERBOSE</string>' >> "$plist"
			echo '		<string>VERSION</string>' >> "$plist"
			echo '		<string>WEIGHT</string>' >> "$plist"
			echo '		<string>WHILE</string>' >> "$plist"
			echo '		<string>WMP_AREAS</string>' >> "$plist"
			echo '		<string>WMP_LINKS</string>' >> "$plist"
			echo '		<string>WRITE_ASCII</string>' >> "$plist"
			echo '		<string>WRITE_ASCII_LIST</string>' >> "$plist"
			echo '		<string>WRITE_ASCII_TERMINATE</string>' >> "$plist"
			echo '		<string>WRITE_ASCII_TERMINATED</string>' >> "$plist"
			echo '		<string>WRITE_ASCIIE</string>' >> "$plist"
			echo '		<string>WRITE_ASCIIL</string>' >> "$plist"
			echo '		<string>WRITE_ASCIIT</string>' >> "$plist"
			echo '		<string>WRITE_BYTE</string>' >> "$plist"
			echo '		<string>WRITE_EVALUATED_ASCII</string>' >> "$plist"
			echo '		<string>WRITE_FILE</string>' >> "$plist"
			echo '		<string>WRITE_LONG</string>' >> "$plist"
			echo '		<string>WRITE_SHORT</string>' >> "$plist"
			echo '	</array>' >> "$plist"
			echo '	<key>BBLMLanguageCode</key>' >> "$plist"
			echo '	<string>WeiD</string>' >> "$plist"
			echo '	<key>BBLMLanguageDisplayName</key>' >> "$plist"
			echo '	<string>WeiDU</string>' >> "$plist"
			echo '	<key>BBLMScansFunctions</key>' >> "$plist"
			echo '	<false/>' >> "$plist"
			echo '	<key>BBLMSuffixMap</key>' >> "$plist"
			echo '	<array>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.baf</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.d</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.tp2</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.tpa</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.tph</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.tpp</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '		<dict>' >> "$plist"
			echo '			<key>BBLMLanguageSuffix</key>' >> "$plist"
			echo '			<string>.tra</string>' >> "$plist"
			echo '		</dict>' >> "$plist"
			echo '	</array>' >> "$plist"
			echo '	<key>Language Features</key>' >> "$plist"
			echo '	<dict>' >> "$plist"
			echo '		<key>Close Block Comments</key>' >> "$plist"
			echo '		<string>*/</string>' >> "$plist"
			echo '		<key>Close Parameter Lists</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Close Statement Blocks</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Close Strings 1</key>' >> "$plist"
			echo '		<string>~</string>' >> "$plist"
			echo '		<key>Close Strings 2</key>' >> "$plist"
			echo '		<string>"</string>' >> "$plist"
			echo '		<key>Comment Pattern</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>End-of-line Ends Strings 1</key>' >> "$plist"
			echo '		<false/>' >> "$plist"
			echo '		<key>End-of-line Ends Strings 2</key>' >> "$plist"
			echo '		<false/>' >> "$plist"
			echo '		<key>Escape Char in Strings 1</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Escape Char in Strings 2</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Function Pattern</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Identifier and Keyword Character Class</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Identifier and Keyword Characters</key>' >> "$plist"
			echo '		<string>0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz</string>' >> "$plist"
			echo '		<key>Open Block Comments</key>' >> "$plist"
			echo '		<string>/*</string>' >> "$plist"
			echo '		<key>Open Line Comments</key>' >> "$plist"
			echo '		<string>//</string>' >> "$plist"
			echo '		<key>Open Parameter Lists</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Open Statement Blocks</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Open Strings 1</key>' >> "$plist"
			echo '		<string>~</string>' >> "$plist"
			echo '		<key>Open Strings 2</key>' >> "$plist"
			echo '		<string>"</string>' >> "$plist"
			echo '		<key>Prefix for Functions</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Prefix for Procedures</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Skip Pattern</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>String Pattern</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Terminator for Prototypes 1</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '		<key>Terminator for Prototypes 2</key>' >> "$plist"
			echo '		<string></string>' >> "$plist"
			echo '	</dict>' >> "$plist"
			echo '</dict>' >> "$plist"
			echo '</plist>' >> "$plist"
			chmod 644 "$plist"
		else
			if test -e "$plist"; then rm -f "$plist" 2>/tmp/weidu_install_errors; fi
		fi
	done
}

########################################################################
##
##  LET'S GET THIS THING STARTED
##  Ok, all the commands and functions are made, now we just have to
##  tell it to go.  Ok - go!
##
########################################################################

clear
command_path=${0%/*}
cd "$command_path"
command=0
command_history=()
where_do_things_go
version_weidu=`check_version "$command_path/weidu-mac"`
size_weidu=`ls -l weidu-mac | awk '{print $5}'`
size_sox=`ls -l sox | awk '{print $5}'`
size_tispack=`ls -l tispack | awk '{print $5}'`
size_tisunpack=`ls -l tisunpack | awk '{print $5}'`
tmp="/tmp/weidu_install"
menus WELCOME
menus QUIT

